SystemOrganization addCategory: #'GT-Examples'!
SystemOrganization addCategory: 'GT-Examples-Announcements'!
SystemOrganization addCategory: 'GT-Examples-Core'!
SystemOrganization addCategory: 'GT-Examples-Exceptions'!
SystemOrganization addCategory: 'GT-Examples-Help'!
SystemOrganization addCategory: 'GT-Examples-Organization'!
SystemOrganization addCategory: 'GT-Examples-Runtime'!
SystemOrganization addCategory: 'GT-Examples-Subjects'!
SystemOrganization addCategory: 'GT-Examples-Testing'!

Trait named: #TGTExampleIcon
	uses: {}
	category: 'GT-Examples-Organization'!

!TGTExampleIcon methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 14:50'!
basicForm
	^ self subclassResponsibility! !

!TGTExampleIcon methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/19/2016 14:50'!
form
	^ self exists
		ifTrue: [ self basicForm ]
		ifFalse: [ nil ]! !

!TGTExampleIcon methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/19/2016 14:50'!
gtInspectorFormIn: composite
	<gtInspectorPresentationOrder: 0>
	self exists ifFalse: [ ^ self ].
	
	self form gtInspectorFormIn: composite! !

Trait named: #TGTExampleMethod
	uses: {}
	category: 'GT-Examples-Organization'!

!TGTExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/19/2016 14:54'!
gtSpotterPreviewIn: aComposite
	<spotterPreview: 1>
	self exists ifFalse: [ ^ self ].
	
	aComposite pharoMethod
		title: [ self gtDisplayString ]; 
		display: [ :entity | entity method sourceCode ];
		smalltalkClass: [ self method methodClass ];
		entity: self! !

!RPackageTag methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 13:27'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self package methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/15/2015 16:56'!
copyWith: anException
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copyWith: anException);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'AndreiChis 10/15/2015 00:15'!
copyWithAll: aCollection
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copy
			addAll: aCollection;
			yourself);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 17:43'!
signaledBy: anException
	(anException isNil and: [ exceptions isEmptyOrNil not ]) ifTrue: [ 
		^ (GTExampleExpectedExceptionNotRaised new 
			expectedError: self;
			exception: anException;
			yourself) signal ].
	(anException notNil and: [ (exceptions includes: anException class) not ]) ifTrue: [ 
		^ (GTExampleUnexpectedExceptionRaised new
			unexpectedError: anException;
			yourself) signal ]! !

Object subclass: #GTExample
	uses: TGTExampleMethod
	instanceVariableNames: 'methodClass selector label description form exceptions subjects children properties after problems extensions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Core'!

!GTExample commentStamp: 'TudorGirba 9/6/2015 11:03' prior: 0!
I represent the concept of an example.

I am constructed out of a method that returns an object. I hold the meta-information about that method and I know how to construct the concrete resulting object that is returned by the method. 

I can depend on other examples, and other examples can depend on me. When I depend on another example, the code that produces me uses the result of the other example to construct me.

The class for which I represent the example is called my subject.

The class that offers the method out of which I am constructed is the provider.

I also know how to handle possible exceptions, which makes me a great candidate for encoding assumptions that are otherwise expressed as classic tests.!

GTExample class
	uses: TGTExampleMethod classTrait
	instanceVariableNames: ''!

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
= anExample
	^ self class = anExample class 
		and: [ self methodClass = anExample methodClass 
			and: [ self selector = anExample selector ] ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:32'!
addChild: aGTDependency
	children add: aGTDependency! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:32'!
addExtension: anExtension
	extensions add: anExtension! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 11:01'!
addProblem: aProblem
	problems add: aProblem! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:32'!
addSubject: aGTExampleSubject
	subjects add: aGTExampleSubject asGTExampleSubject! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 20:44'!
after
	^ after! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:29'!
after: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by the given selector, to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self method: GTExampleMethodAfter forSelector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 4/10/2016 18:59'!
afterArguments
	self after ifNil: [ ^ #() ].
	^ self after arguments! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:35'!
afterClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by (aClassOrClassName >> aSelector), to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self method: GTExampleMethodAfter forClassNamed: aClassOrClassName selector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 21:38'!
allDependencies
	| examples |
	examples := Set new.
	(self traverser
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 21:35'!
allDependents
	| examples |
	examples := Set new.
	(self traverser reversed
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 20:54'!
arguments
	^ children asArray! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 11:45'!
classNamed: aClassOrClassName
	^ aClassOrClassName isClass 
		ifTrue: [ aClassOrClassName ]
		ifFalse: [ Smalltalk classNamed: aClassOrClassName asString ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
classNamed: aClassName do: aBlock
	^ self classNamed: aClassName do: aBlock ifNone: [ 
		(GTExampleDeclaredClassMissing new
			example: self;
			classNamed: aClassName;
			yourself) signal ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:04'!
classNamed: aClassName do: aBlock1 ifNone: aBlock2
	^ (self classNamed: aClassName)
		ifNotNil: aBlock1
		ifNil: aBlock2! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 9/2/2015 22:19'!
debug
	^ self debugger result! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
debugger
	^ GTExampleDebugger new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:02'!
dependencies
	^ self arguments collect: [ :each | each gtExample ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:27'!
dependenciesOrArguments
	^ self arguments collect: [ :each | each gtExample ifNil: [ each ] ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:30'!
depends: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	self addChild: (self method: GTExampleMethod forSelector: aSelector)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:31'!
dependsAll: aString
	<gtExamplePragma>
	<description: 'This example depends on all given selectors found in the given string (list separated by  .,;). Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	self substrings: aString do: [ :each | 
		self depends: aString ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/11/2016 17:51'!
dependsClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given class and selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	children add: (self method: GTExampleMethod forClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
description
	^ description! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:40'!
description: aString
	<gtExamplePragma>
	<description: 'A full description of this example'>
	
	description := aString! !

!GTExample methodsFor: 'TODO' stamp: 'StefanReichhart 7/4/2016 19:39'!
directDependents
	<todo: 'better name -> #myDependents and #dependents is taken - wtf!!'>
	^ (GTExampleOrganizer instance examplesSelect: [ :each | each hasDependency: self ]) asSet asArray! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
evaluator
	^ GTExampleEvaluator new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/29/2015 10:58'!
exceptions
	^ exceptions! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:31'!
exists
	^ self methodClass includesSelector: self selector! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 14:47'!
form
	^ form! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 14:21'!
form: anIcon
	form := anIcon! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 15:52'!
gtDebuggerSourceIn: composite
	<gtDebuggerPresentationOrder: 1>
	self exists ifFalse: [ ^ self ].
	^ (self method gtInspectorSourceIn: composite)
		title: 'Method'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:19'!
gtDisplayOn: aStream
	self label isEmptyOrNil 
		ifTrue: [ 
			aStream 
				print: self methodClass; 
				nextPutAll: ' >> '; 
				print: self selector ]
		ifFalse: [ 
			aStream nextPutAll: self label ].
		
	self gtDisplayValidityOn: aStream! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:17'!
gtDisplayValidityOn: aStream
	self isValid ifTrue: [ ^ self ].
	
	aStream nextPutAll: ' [invalid]'! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 3/28/2016 19:28'!
gtExample
	<gtExamplePragma>
	<description: 'This method is turned into a gtExample if provided'>
	
	^ self! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/3/2016 15:03'!
gtExample: anObject
	<toBeDeprecated: 'Please define your example-label (or description) using the pragma #label: or #description:'>
	self label: anObject.
	self gtExample! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:19'!
gtExamplesContained
	^ { self }! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:02'!
gtExamplesFactory
	^ self method gtExamplesFactory! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/6/2016 13:30'!
gtInspectorActionReturnValue
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ :p | p selection: (self runThenDo: [ :r | r returnValue ] onErrorDo: [ :r | r ]) ];
		icon: (Smalltalk ui icons iconNamed: #glamorousGo);
		title: 'Evaluate and inspect return-value'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/13/2016 09:58'!
gtInspectorActionSubjects
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ GTExampleNavigation browseAllExampleSubjects: { self } ];
		icon: (Smalltalk ui icons iconNamed: #glamorousBookmark);
		title: 'Subjects'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/21/2016 17:12'!
gtInspectorActions
	^ super gtInspectorActions 
		addAll: self gtInspectorCustomActions;
		yourself! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:34'!
gtInspectorAfterIn: composite
	<gtInspectorPresentationOrder: 3>
	self hasAfter ifFalse: [ ^ self ].
	^ ({ self after } gtInspectorItemsIn: composite)
			title: 'After'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/21/2016 17:39'!
gtInspectorCustomActions
	self hasExtensions ifFalse: [ ^ { } ].
	
	^ Array streamContents: [ :stream | 
		extensions do: [ :each | 
			(each gtInspectorActionFor: self) ifNotNil: [ :action |
				stream nextPut: action ] ] ] ! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/21/2016 14:51'!
gtInspectorCustomExtensionsFor: composite
	<gtInspectorPresentationOrder: 99>
	self hasExtensions ifFalse: [ ^ self ].
	
	extensions do: [ :extension |
		extension gtInspectorFor: composite example: self ]! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:33'!
gtInspectorDependenciesIn: composite
	<gtInspectorPresentationOrder: 1>
	self hasDependencies ifFalse: [ ^ self ].
	^ (self dependenciesOrArguments gtInspectorItemsIn: composite)
			title: 'Dependencies'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:33'!
gtInspectorDependentsIn: composite
	<gtInspectorPresentationOrder: 2>
	self hasDependents ifFalse: [ ^ self ].
	^ (self directDependents gtInspectorItemsIn: composite)
			title: 'Dependents'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 15:52'!
gtInspectorMethodIn: composite
	<gtInspectorPresentationOrder: 0>
	self exists ifFalse: [ ^ self ].
	^ (self method gtInspectorSourceIn: composite)
		title: 'Method'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:35'!
gtInspectorProblemsIn: composite
	<gtInspectorPresentationOrder: 5>
	self hasProblems ifFalse: [ ^ self ].
	^ (self problems gtInspectorItemsIn: composite)
			title: 'Problems'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 19:45'!
gtInspectorRoassalIn: composite
	<gtInspectorPresentationOrder: 1>
	self hasDependencies ifFalse: [ ^ self ].
	(Smalltalk classNamed: #GLMRoassal2Presentation) ifNil: [ ^ self ].
	^ composite roassal2
		title: 'Graph';
		painting: [ :view :example | 
			(GTExampleGraphPainter new 
				example: example;
				view: view;
				yourself) paint ];
		yourself! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/14/2016 12:24'!
gtInspectorSourceIn: composite
	<gtInspectorPresentationOrder: 0>
	^ (self method gtInspectorSourceIn: composite)
		title: 'Source'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:36'!
gtInspectorSubjectsIn: composite
	<gtInspectorPresentationOrder: 4>
	self hasSubjects ifFalse: [ ^ self ].
	^ (self subjects gtInspectorItemsIn: composite)
			title: 'Subjects'! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterAfterFor: aStep
	<spotterOrder: 3>
	self hasAfter ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example After';
		items: [ { self after } ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/21/2016 14:40'!
gtSpotterCustomExtensionsFor: aStep
	<spotterOrder: 99>
	self hasExtensions ifFalse: [ ^ self ].
	
	extensions do: [ :extension |
		extension gtSpotterFor: aStep example: self ]! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterDependenciesFor: aStep
	<spotterOrder: 1>
	self hasDependencies ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependencies';
		items: [ self dependenciesOrArguments ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterDependentsFor: aStep
	<spotterOrder: 2>
	self hasDependents ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependents';
		items: [ self directDependents ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterGTExampleSubjectsFor: aStep
	<spotterOrder: 4>
	self hasSubjects ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Subjects';
		items: [ self subjects asArray ];
		itemName: [ :subject | subject gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterMethodFor: aStep
	<spotterOrder: 0>
	self exists ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Method';
		items: [ Array with: self method ];
		itemName: [ :aSource | aSource gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 18:32'!
gtSpotterProblemsFor: aStep
	<spotterOrder: 5>
	self hasProblems ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Problems';
		items: [ self problems ];
		itemName: [ :problem | problem gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:09'!
hasAfter
	^ after notNil! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 17:48'!
hasAfter: anAfterMethod
	^ after = anAfterMethod! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:09'!
hasAfterArguments
	^ self hasAfter
		and: [ self after hasArguments ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:06'!
hasArguments
	^ self arguments notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:04'!
hasDependencies
	^ self arguments notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 18:24'!
hasDependency: aGTExample
	^ self arguments anySatisfy: [ :each | each matchesMethod: aGTExample ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:45'!
hasDependent: anExample
	^ self directDependents anySatisfy: [ :each | each matchesExample: anExample ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:45'!
hasDependents
	^ self directDependents notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/21/2016 13:58'!
hasExtensions
	^ extensions isEmptyOrNil not! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/13/2016 18:21'!
hasInternalProblems
	^ problems notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/15/2016 19:59'!
hasLiteralThorough: aLiteral
	^ self methodClass = aLiteral
		or: [ self selector = aLiteral
			or: [ (self label notNil and: [ self label includesSubstring: aLiteral asString caseSensitive: false ])
				or: [ (self description notNil and: [ self description includesSubstring: aLiteral asString caseSensitive: false ])
					or: [ (self subjects anySatisfy: [ :each | each hasLiteralThorough: aLiteral ])
						or: [ self exists and: [ self method hasLiteralThorough: aLiteral ] ] ] ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 18:45'!
hasProblems
	^ problems notEmpty
		or: [ self hasValidArguments not
			or: [  self hasValidAfter not
				or: [ self hasValidDependencies not ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/21/2015 20:33'!
hasRelationships
	^ self hasDependents
		or: [ self hasDependencies ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 20:20'!
hasSubject: aSubject
	^ self subjects anySatisfy: [ :each | 
		each == aSubject 
			or: [ each = aSubject ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 8/6/2016 22:05'!
hasSubjectMatching: aStringOrSubject
	^ (self hasSubject: aStringOrSubject)
		or: [ self subjects anySatisfy: [ :each | 
			each hasLiteralThorough: aStringOrSubject ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:26'!
hasSubjects
	^ self subjects notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 18:46'!
hasValidAfter
	^ self hasAfter not
		or: [ self after isValid ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:07'!
hasValidArguments
	^ self arguments size = self method numArgs! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 20:32'!
hasValidDependencies
	^ self hasValidArguments 
		and: [ self hasDependencies not
			or: [ self dependencies allSatisfy: [ :each | 
				each notNil and: [ each exists ] ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/21/2016 14:39'!
hasValidExtensions
	^ self hasExtensions not
		or: [ extensions allSatisfy: [ :each | 
				each notNil and: [ each isValid ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
hash
	^ (self class hash 
		bitXor: self methodClass hash) 
			bitXor: self selector hash! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:21'!
icon
	^ form ifNotNil: [ form form ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/19/2016 15:04'!
icon: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must be implemented by the currently active theme'>

	self iconThemeSelector: aSelector! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:26'!
iconBase64: aString
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The icon form is described by the given base64-encoded string'>
	
	form := GTExampleFormIcon fromBase64String: aString! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:26'!
iconBytes: aByteArray
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The icon form is described by the given byte-array'>
	
	form := GTExampleFormIcon fromByteArray: aByteArray! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:33'!
iconClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The method described by the given class and selector must return an icon form'>
	
	form := self method: GTExampleMethodIcon forClassNamed: aClassOrClassName selector: aSelector! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:31'!
iconSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must return an icon form'>
	
	form := (self method: GTExampleMethodIcon forSelector: aSelector)
		converter: #convertFromByteArray:;
		yourself! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:31'!
iconSelectorBase64: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The icon form is described by the base64 string returned by the given selector'>
	
	form := (self method: GTExampleMethodIcon forSelector: aSelector)
		converter: #convertFromBase64String:;
		yourself! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:28'!
iconThemeSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must be implemented by the currently active theme'>
	
	form := GTExampleThemeIcon fromSelector: aSelector! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 10:59'!
ignoreTest
	^ self propertyAt: #noTest ifAbsent: [ false ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 4/24/2016 17:16'!
ignoreTestReport
	^ self propertyAt: #noTestReport ifAbsent: [ false ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/21/2016 14:46'!
initialize
	super initialize.
	
	children := OrderedCollection new.
	subjects := Set new.
	problems := Set new.
	exceptions := ExceptionSet new.	
	properties := IdentityDictionary new.
	extensions := Set new.! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/11/2016 09:40'!
initializeFromMethod: aCompiledMethod usingFactory: aGTExampleFactory
	methodClass := aCompiledMethod methodClass.
	selector := aCompiledMethod selector.
	aGTExampleFactory initializeExample: self fromMethod: aCompiledMethod! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:34'!
inspector: aSelector
	<gtExamplePragma>
	<description: 'A method providing a custom GTInspector-extensions for this particular example'>
	
	self addExtension: (self method: GTExampleMethodInspectorExtension forSelector: aSelector)! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:34'!
inspectorAction: aSelector
	<gtExamplePragma>
	<description: 'A method providing a custom GTInspector-action for this particular example'>
	
	self addExtension: (self method: GTExampleMethodInspectorActionExtension forSelector: aSelector)! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:35'!
inspectorActionClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by (aClassOrClassName >> aSelector), providing a custom GTInspector-actions for this particular example'>
	
	self addExtension: (self method: GTExampleMethodInspectorActionExtension forClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:35'!
inspectorClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by (aClassOrClassName >> aSelector), providing custom GTInspector-extensions for this particular example'>
	
	self addExtension: (self method: GTExampleMethodInspectorExtension forClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/4/2016 20:44'!
invalidArguments
	^ self arguments reject: [ :each | each isValid ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/14/2016 18:10'!
invalidDependencies
	^ self arguments reject: [ :each | each isValid or: [ each exists ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:57'!
isGTExample
	^ true! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:52'!
isReferencing: aLiteral
	^ (self hasSubject: aLiteral)
		or: [ (self subjects anySatisfy: [ :subject | subject hasLiteralThorough: aLiteral ])
			or: [ (self exists and: [ self method hasLiteralThorough: aLiteral ])
				or: [ (self methodClass name asString = aLiteral name asString) ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 22:53'!
isValid
	^ self hasProblems not
		and: [ self traverser canProcess ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
label
	^ label! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:39'!
label: aString
	<gtExamplePragma>
	<description: 'A short identifier = name of this example'>
	
	label := aString! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/5/2016 16:11'!
lastResult
	^ GTExampleOrganizer instance resultAt: self ifAbsent: [ nil ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/5/2016 16:13'!
lastReturnValue
	^ self lastResult ifNotNil: [ :lr | lr returnValue ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:30'!
matchesMethod: aCompiledMethod
	^ aCompiledMethod class = CompiledMethod
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:34'!
method
	^ self methodClass >> self selector! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 17:50'!
method: aMethodClass forClass: aClass named: aClassName selector: aSelector
	^ aMethodClass new
		methodClass: aClass;
		methodClassName: aClassName;
		selector: aSelector asSymbol;
		yourself! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 17:50'!
method: aMethodClass forClass: aClass selector: aSelector
	^ self method: aMethodClass forClass: aClass named: aClass name selector: aSelector! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 17:50'!
method: aMethodClass forClassNamed: aClassOrClassName selector: aSelector
	^ self classNamed: aClassOrClassName 
		do: [ :class | self method: aMethodClass forClass: class named: aClassOrClassName selector: aSelector ] 
		ifNone: [ self method: aMethodClass forClass: nil named: aClassOrClassName selector: aSelector ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/11/2016 17:51'!
method: aMethodClass forSelector: aSelector
	^ self method: aMethodClass forClass: self methodClass selector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/14/2016 18:16'!
methodArgumentNames
	^ self exists
		ifTrue: [ self method argumentNames ]
		ifFalse: [ #() ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
methodClass
	^ methodClass! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:19'!
methodReference
	^ self method asRingDefinition! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 3/28/2016 19:26'!
noTest
	<gtExamplePragma>
	<description: 'Do not run this example while running SUnit tests'>
	
	self propertyAt: #noTest put: true! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 4/24/2016 17:17'!
noTestReport
	<gtExamplePragma>
	<description: 'Do not run this example while running HDTestReport (not yet implemented)'>
	
	self propertyAt: #noTestReport put: true! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 10:58'!
problems
	" some problems are static -> #problems / other problems are dynamic (and changing) and not stored in #problems -> arguments+examples "
	^ (problems , self invalidArguments) asArray! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 09:43'!
propertyAt: aKey
	^ properties at: aKey! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 09:43'!
propertyAt: aKey ifAbsent: aBlock
	^ properties at: aKey ifAbsent: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 09:43'!
propertyAt: aKey ifAbsentPut: aBlock
	^ properties at: aKey ifAbsentPut: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 09:43'!
propertyAt: aKey put: aValue
	^ properties at: aKey put: aValue! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:54'!
provider 
	^ self methodClass isMeta
		ifTrue: [ self methodClass theNonMetaClass ]
		ifFalse: [ self methodClass theNonMetaClass new ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:36'!
raises: aClassOrClassName
	<gtExamplePragma>
	<description: 'This example is expected to raise the given exception. The example (as a test) will fail if the exception is not raised.'>
	
	self classNamed: aClassOrClassName do: [ :class | 
		self exceptions add: class ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:37'!
raisesAny: aString
	<gtExamplePragma>
	<description: 'This example is expected to raise any of the given exceptions (list separated by  .,;). The example (as a test) will fail if none of the exceptions is raised.'>
	
	self substrings: aString do: [ :each | 
		self raises: each ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/11/2016 17:22'!
reinitialize
	| method |
	(method := self method) ifNil: [ ^ self ].
	
	self initialize.
	method gtExamplesFactory initializeExample: self fromMethod: method! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 19:05'!
result
	^ self run! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/2/2015 12:00'!
returnValue
	^ self result returnValue! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 7/2/2016 21:55'!
run
	^ self evaluator result ! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 7/6/2016 13:27'!
runThenDo: aBlock1 onErrorDo: aBlock2
	| result |
	^ (result := self run) isSuccess 
		ifTrue: [ aBlock1 value: result ]
		ifFalse: [ aBlock2 value: result ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
selector
	^ selector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 5/13/2016 11:02'!
signalableExceptions
	^ self exceptions copyWithAll: { Halt. Error. Exit. UnhandledError. TestFailure. Deprecation. }! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 18:37'!
sourceCode
	^ self method sourceCode! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:37'!
spotter: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by aSelector, providing a custom GTSpotter-extension for this particular example'>
	
	self addExtension: (self method: GTExampleMethodSpotterExtension forSelector: aSelector)! !

!GTExample methodsFor: '*GT-Examples-UI-PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:37'!
spotterClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method, implemented by (aClassOrClassName >> aSelector), providing a custom GTSpotter-extension for this particular example'>
	
	self addExtension: (self method: GTExampleMethodSpotterExtension forClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 9/9/2015 20:10'!
spotterPreviewIn: aComposite inContext: aSpotter
	^ self gtSpotterPreviewIn: aComposite! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/19/2016 15:05'!
subject: aClassOrClassName
	<gtExamplePragma>
	<description: 'Add a class-subject to the example'>
	
	self subjectClass: aClassOrClassName! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/19/2016 15:05'!
subjectClass: aClassOrClassName
	<gtExamplePragma>
	<description: 'Add a class-subject to the example'>
	
	self addSubject: (self subjectForClassNamed: aClassOrClassName)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/19/2016 15:05'!
subjectClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'Add a method-subject to the example'>
	
	self addSubject: (self method: GTExampleMethodSubject forClassNamed: aClassOrClassName selector: aSelector)! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 8/13/2016 15:53'!
subjectForClassNamed: aClassName
	^ self classNamed: aClassName 
		do: [ :class | GTExampleClassSubject fromClass: class ] 
		ifNone: [ GTExampleClassSubject fromClassNamed: aClassName ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:38'!
subjectHelp: bookName
	<gtExamplePragma>
	<description: 'Add a help/topic-subject to the example'>
	
	self subjectHelp: bookName title: nil! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 15:59'!
subjectHelp: bookName title: title
	<gtExamplePragma>
	<description: 'Add a help/topic-subject to the example'>
	
	self addSubject: (GTExampleHelpSubject fromBook: bookName title: title)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 15:45'!
subjectKeyword: aStringOrSymbol
	<gtExamplePragma>
	<description: 'Add a keyword to the example'>
	
	self addSubject: (GTExampleKeywordSubject fromString: aStringOrSymbol)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 12:21'!
subjectKeywords: anArrayOrString
	<gtExamplePragma>
	<description: 'Add multiple keywords to the example'>
	
	anArrayOrString gtExampleSubstrings do: [ :each |
		self subjectKeyword: each ]! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:02'!
subjectPackage: packageName
	<gtExamplePragma>
	<description: 'Add a package-subject to the example'>
	
	self addSubject: (GTExamplePackageSubject fromPackageNamed: packageName)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 15:44'!
subjectSelector: aSelector
	<gtExamplePragma>
	<description: 'Add a selector subject to the example'>
	
	self addSubject: (GTExampleSelectorSubject fromString: aSelector)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 16:04'!
subjectTag: tagName
	<gtExamplePragma>
	<description: 'Add a tag-subject to the example'>
	
	self addSubject: (GTExampleTagSubject fromTagNamed: tagName)! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 8/13/2016 15:43'!
subjectText: aString
	<gtExamplePragma>
	<description: 'Add a text-subject to the example'>
	
	self addSubject: (GTExampleTextSubject fromText: aString)! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 10:58'!
subjects
	^ subjects asArray! !

!GTExample methodsFor: 'PRAGMA-SELECTORS' stamp: 'StefanReichhart 7/21/2016 18:38'!
subjectsAll: aString
	<gtExamplePragma>
	<description: 'Add multiple class-subjects (list separated by  .,;) to the example'>
	
	self substrings: aString do: [ :each | 
		self subject: each ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 8/13/2016 12:22'!
substrings: aString do: aBlock
	aString gtExampleSubstrings do: [ :each | 
		aBlock value: each ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
traverser
	^ GTExampleTraverser new 
		example: self;
		yourself! !

Object subclass: #GTExampleAspect
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

GTExampleAspect subclass: #GTAbstractExampleMethod
	uses: TGTExampleMethod
	instanceVariableNames: 'methodClassName methodClass selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

GTAbstractExampleMethod class
	uses: TGTExampleMethod classTrait
	instanceVariableNames: ''!

!GTAbstractExampleMethod class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:09'!
fromClass: aClass selector: aSelector
	^ self new 
		methodClass: aClass;
		methodClassName: aClass name;
		selector: aSelector;
		yourself! !

!GTAbstractExampleMethod class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:09'!
fromClassNamed: aString selector: aSelector
	^ self new 
		methodClass: nil;
		methodClassName: aString asString;
		selector: aSelector;
		yourself! !

!GTAbstractExampleMethod class methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:47'!
isAbstract
	^ self == GTAbstractExampleMethod! !

!GTAbstractExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:43'!
= anExampleMethod
	^ self class = anExampleMethod class 
		and: [ self methodClassName = anExampleMethod methodClassName 
			and: [ self selector = anExampleMethod selector ] ]! !

!GTAbstractExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 15:58'!
addToSelectorEnvironment: aSelectorEnvironment
	self exists ifFalse: [ ^ self ].
	
	aSelectorEnvironment addClass: self methodClass selector: self selector! !

!GTAbstractExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 13:38'!
exists
	^ self methodClassName notNil
		and: [ self methodClass notNil
			and: [ self methodClass includesSelector: self selector ] ]! !

!GTAbstractExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock
		value: #'Browse Method ' ,  self method gtDisplayString
		value: [ self method browse ]! !

!GTAbstractExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:11'!
gtDisplayOn: aStream
	aStream 
		nextPutAll: self methodClassName asString; 
		nextPutAll: ' >> #'; 
		nextPutAll: self selector asString.
		
	self gtDisplayValidityOn: aStream! !

!GTAbstractExampleMethod methodsFor: '*GT-Examples-UI' stamp: ''!
gtSpotterPreviewIn: aComposite
	<spotterPreview: 1>
	self exists ifFalse: [ ^ self ].
	
	aComposite pharoMethod
		title: [ self gtDisplayString ]; 
		display: [ :entity | entity method sourceCode ];
		smalltalkClass: [ self method methodClass ];
		entity: self! !

!GTAbstractExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:57'!
hasLiteralThorough: aLiteral
	^ self methodClassName = aLiteral 
		or: [ self selector = aLiteral
			or: [ self exists 
				and: [ self method hasLiteralThorough: aLiteral ] ] ]! !

!GTAbstractExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:43'!
hash
	^ (self class hash 
		bitXor: self methodClassName hash)
			bitXor: self selector hash! !

!GTAbstractExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 13:38'!
matchesMethod: aCompiledMethod
	^ self exists
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTAbstractExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/11/2016 15:22'!
method
	^ self exists 
		ifTrue: [ self methodClass >> self selector ]
		ifFalse: [ nil ]! !

!GTAbstractExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/14/2016 18:16'!
methodArgumentNames
	^ self exists
		ifTrue: [ self method argumentNames ]
		ifFalse: [ #() ]! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 17:25'!
methodClass
	^ methodClass ifNil: [ methodClass := Smalltalk classNamed: methodClassName ]
		! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:24'!
methodClass: anObject
	methodClass := anObject! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:24'!
methodClassName
	^ methodClassName! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:24'!
methodClassName: anObject
	methodClassName := anObject! !

!GTAbstractExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/19/2016 21:38'!
provider 
	self exists ifFalse: [ ^ nil ].
	^ self methodClass isMeta
		ifTrue: [ self methodClass theNonMetaClass ]
		ifFalse: [ self methodClass theNonMetaClass new ]! !

!GTAbstractExampleMethod methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub
	" 1) reset -> 2) reinitialize  "
	methodClass := nil! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:24'!
selector
	^ selector! !

!GTAbstractExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:24'!
selector: anObject
	selector := anObject! !

GTAbstractExampleMethod subclass: #GTExampleMethod
	instanceVariableNames: 'arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments
	^ arguments ifNil: [ #() ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments: anObject
	arguments := anObject! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/13/2016 16:10'!
asGTExampleSubject
	^ GTExampleMethodSubject fromClass: self methodClass selector: self selector! !

!GTExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 17:54'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock
		value: #'Browse dependency Method ' ,  self method gtDisplayString
		value: [ self method browse ]! !

!GTExampleMethod methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/22/2016 11:41'!
gtDisplayValidityOn: aStream
	self isValid ifTrue: [ ^ self ].
	
	self exists ifFalse: [ ^ aStream nextPutAll: ' [does not exist]' ].
	self hasValidArguments ifFalse: [ ^ aStream nextPutAll: '[invalid arguments]' ].
	aStream nextPutAll: ' [invalid]'! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/19/2016 13:01'!
gtExample
	^ self hasExample 
		ifTrue: [ self method gtExample ]
		ifFalse: [ nil ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:13'!
hasArguments
	^ self arguments notEmpty! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/19/2016 13:02'!
hasExample
	^ self exists
		and: [ self method isGTExampleMethod ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:07'!
hasValidArguments
	^ self exists 
		and: [ self arguments size = self method numArgs ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 20:40'!
isValid
	^ super isValid 
		and: [ self hasValidArguments ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/19/2016 13:02'!
matchesExample: aGTExample
	^ self exists 
		and: [ aGTExample isGTExample
			and: [ aGTExample methodClass = self methodClass
				and: [ aGTExample selector = self selector ] ] ]! !

GTAbstractExampleMethod subclass: #GTExampleMethodAfter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

!GTExampleMethodAfter methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 17:47'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock
		value: #'Browse after Method ' ,  self method gtDisplayString
		value: [ self method browse ]! !

!GTExampleMethodAfter methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 17:47'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 102>
	aStep listProcessor
		title: 'Examples with after Method ', self gtDisplayString;
		filter: GTNullFilter item: [ :filter :context | 
			GTExampleOrganizer instance examplesDo: [ :cn :s :example |
				(example hasAfter: self) ifTrue: [ filter value: example ] ] ];
		wantsToDisplayOnEmptyQuery: true! !

GTAbstractExampleMethod subclass: #GTExampleMethodIcon
	uses: TGTExampleIcon
	instanceVariableNames: 'converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

GTExampleMethodIcon class
	uses: TGTExampleIcon classTrait
	instanceVariableNames: ''!

!GTExampleMethodIcon class methodsFor: 'converters' stamp: 'StefanReichhart 8/13/2016 16:30'!
convertFromBase64String: aString
	^ Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: aString readStream)! !

!GTExampleMethodIcon class methodsFor: 'converters' stamp: 'StefanReichhart 8/13/2016 16:30'!
convertFromByteArray: aByteArray
	^ self convertFromByteArray: aByteArray extent: 16@16 depth: 32 offset: 0@0! !

!GTExampleMethodIcon class methodsFor: 'converters' stamp: 'StefanReichhart 8/13/2016 16:30'!
convertFromByteArray: aByteArray extent: anExtent depth: aDepth offset: anOffset
	^ Form
		extent: anExtent
		depth: aDepth
		fromArray: aByteArray 
		offset: anOffset! !

!GTExampleMethodIcon methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 13:46'!
basicForm
	^ self class 
		perform: self converter 
		withArguments: { self provider perform: self selector }! !

!GTExampleMethodIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:37'!
converter
	^ converter! !

!GTExampleMethodIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:37'!
converter: anObject
	converter := anObject! !

!GTExampleMethodIcon methodsFor: 'testing' stamp: 'StefanReichhart 8/19/2016 11:39'!
exists
	^ super exists
		and: [ self converter notNil
			and: [ self class class includesSelector: self converter ] ]! !

!GTExampleMethodIcon methodsFor: 'accessing-dynamic' stamp: ''!
form
	^ self exists
		ifTrue: [ self basicForm ]
		ifFalse: [ nil ]! !

!GTExampleMethodIcon methodsFor: '*GT-Examples-UI' stamp: ''!
gtInspectorFormIn: composite
	<gtInspectorPresentationOrder: 0>
	self exists ifFalse: [ ^ self ].
	
	self form gtInspectorFormIn: composite! !

GTAbstractExampleMethod subclass: #GTExampleMethodSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleMethodSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/4/2016 13:25'!
asGTExampleSubject
	^ self! !

!GTExampleMethodSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock
		value: #'Browse subject Method ' ,  self method gtDisplayString
		value: [ self method browse ]! !

!GTExampleMethodSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 13:51'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 102>
	aStep listProcessor
		title: 'Examples with Subject ', self gtDisplayString;
		filter: GTNullFilter item: [ :filter :context | 
			GTExampleOrganizer instance examplesDo: [ :cn :s :example |
				(example hasSubject: self) ifTrue: [ filter value: example ] ] ];
		wantsToDisplayOnEmptyQuery: true! !

!GTExampleAspect class methodsFor: 'testing' stamp: 'TudorGirba 8/20/2016 00:29'!
isAbstract
	^ self == GTExampleAspect! !

!GTExampleAspect methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 15:44'!
addToSelectorEnvironment: aSelectorEnvironment! !

!GTExampleAspect methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 16:00'!
exists
	^ false! !

!GTExampleAspect methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 15:44'!
forNautilus: aBlock
	! !

!GTExampleAspect methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/22/2016 11:41'!
gtDisplayValidityOn: aStream
	self isValid ifTrue: [ ^ self ].
	
	self exists ifFalse: [ ^ aStream nextPutAll: ' [does not exist]' ].
	aStream nextPutAll: ' [invalid]'! !

!GTExampleAspect methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:44'!
hasLiteralThorough: aLiteral
	^ false! !

!GTExampleAspect methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:47'!
isAbstract
	^ self class isAbstract! !

!GTExampleAspect methodsFor: 'testing' stamp: 'StefanReichhart 7/15/2016 20:07'!
isValid
	^ self exists! !

!GTExampleAspect methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub! !

GTExampleAspect subclass: #GTExampleIcon
	uses: TGTExampleIcon
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

GTExampleIcon class
	uses: TGTExampleIcon classTrait
	instanceVariableNames: ''!

GTExampleIcon subclass: #GTExampleFormIcon
	instanceVariableNames: 'arguments converter form'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

!GTExampleFormIcon class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/19/2016 11:28'!
fromBase64String: aString
	^ self new
		converter: #convertFromBase64String:;
		arguments: { aString };
		yourself! !

!GTExampleFormIcon class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/19/2016 11:28'!
fromByteArray: aByteArray
	^ self new
		converter: #convertFromByteArray:;
		arguments: { aByteArray };
		yourself! !

!GTExampleFormIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
arguments
	^ arguments! !

!GTExampleFormIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
arguments: anObject
	arguments := anObject! !

!GTExampleFormIcon methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 13:45'!
basicForm
	^ [ GTExampleMethodIcon 
		perform: self converter 
		withArguments: self arguments ] on: Error do: [ nil ]! !

!GTExampleFormIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
converter
	^ converter! !

!GTExampleFormIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
converter: anObject
	converter := anObject! !

!GTExampleFormIcon methodsFor: 'testing' stamp: 'StefanReichhart 7/19/2016 14:39'!
exists
	^ self converter notNil
		and: [ (GTExampleMethodIcon includesSelector: self converter)
			and: [ self form notNil ] ]! !

!GTExampleFormIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 13:45'!
form
	^ form ifNil: [ form := self basicForm ]! !

!GTExampleIcon class methodsFor: 'testing' stamp: 'TudorGirba 8/20/2016 00:29'!
isAbstract
	^ self == GTExampleIcon! !

!GTExampleIcon methodsFor: 'private' stamp: ''!
basicForm
	^ self subclassResponsibility! !

!GTExampleIcon methodsFor: 'accessing-dynamic' stamp: ''!
form
	^ self exists
		ifTrue: [ self basicForm ]
		ifFalse: [ nil ]! !

!GTExampleIcon methodsFor: '*GT-Examples-UI' stamp: ''!
gtInspectorFormIn: composite
	<gtInspectorPresentationOrder: 0>
	self exists ifFalse: [ ^ self ].
	
	self form gtInspectorFormIn: composite! !

GTExampleIcon subclass: #GTExampleThemeIcon
	instanceVariableNames: 'selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

!GTExampleThemeIcon class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:27'!
fromSelector: aSelector
	^ self new 
		selector: aSelector;
		yourself! !

!GTExampleThemeIcon class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:27'!
fromString: aString
	^ self fromSelector: aString asSymbol! !

!GTExampleThemeIcon methodsFor: 'private' stamp: 'StefanReichhart 7/19/2016 14:34'!
basicForm
	^ Smalltalk ui icons iconNamed: self selector ! !

!GTExampleThemeIcon methodsFor: 'testing' stamp: 'StefanReichhart 7/19/2016 14:34'!
exists
	^ (Smalltalk ui icons iconNamed: self selector ifNone: [ nil ]) notNil! !

!GTExampleThemeIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
selector
	^ selector! !

!GTExampleThemeIcon methodsFor: 'accessing' stamp: 'StefanReichhart 7/19/2016 14:24'!
selector: anObject
	selector := anObject! !

GTExampleAspect subclass: #GTExampleSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleSubject commentStamp: 'TudorGirba 8/20/2016 00:27' prior: 0!
I am the abstract of a description of a subject of an example. The concrete subclasses encapsulate the links to the concrete subjects.

For example, the goal of an example might be to exemplify the way an instance of a class X can look like. In this case,  the subject is the class X. Or, the goal of the example might be to instantiate class X and in this case, the subject can be the instantiating method.

The subject information is useful both when understanding the goal of an example, and to link examples to the static entities. For example, if a class has examples associated, these can appear in the static code browser, or in the inspector of the class.!

GTExampleSubject subclass: #GTExampleClassSubject
	instanceVariableNames: 'theClassName theClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleClassSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:49'!
fromClass: aClass
	^ self new 
		theClass: aClass;
		theClassName: aClass name;
		yourself! !

!GTExampleClassSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:54'!
fromClassNamed: aString
	^ self new 
		theClass: nil;
		theClassName: aString asString;
		yourself! !

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:48'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self theClassName = anExampleSubject theClassName ]! !

!GTExampleClassSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:54'!
addToSelectorEnvironment: aSelectorEnvironment
	self exists ifFalse: [ ^ self ].
	
	aSelectorEnvironment addClass: self theClass! !

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 14:01'!
exists
	^ self theClass notNil! !

!GTExampleClassSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock 
		value: #'Browse subject Class ' ,  self theClassName 
		value: [ self theClass browse ]
! !

!GTExampleClassSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:15'!
gtDisplayOn: aStream
	self theClassName gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 15:23'!
hasLiteralThorough: aLiteral
	^ self theClassName = aLiteral
		or: [ self exists 
			and: [ self theClass = aLiteral
				or: [ self theClass methods anySatisfy: [ :method | 
							method selector = aLiteral
								or: [ 	method hasLiteralThorough: aLiteral ] ] ] ] ]! !

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:46'!
hash
	^ self class hash
		bitXor: self theClassName hash! !

!GTExampleClassSubject methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub
	" 1) reset -> 2) reinitialize "
	theClass := nil! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 17:27'!
theClass
	^ theClass ifNil: [ theClass := Smalltalk classNamed: theClassName ]
		! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/22/2016 12:42'!
theClass: anObject
	anObject ifNotNil: [ self theClassName: anObject name ].
	theClass := anObject.
	! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:30'!
theClassName
	^ theClassName! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:30'!
theClassName: anObject
	theClassName := anObject! !

GTExampleClassSubject subclass: #GTExampleSubclassesSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleSubclassesSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/22/2016 12:32'!
addToSelectorEnvironment: aSelectorEnvironment
	self exists ifFalse: [ ^ self ].
	
	super addToSelectorEnvironment: aSelectorEnvironment.
	self theSubclasses do: [ :class | 
		class asGTExampleSubject addToSelectorEnvironment: aSelectorEnvironment ]! !

!GTExampleSubclassesSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/22/2016 12:35'!
gtExamplesContained
	| subjects |
	subjects := self theSubclasses collect: [ :each | each asGTExampleSubject ].
	^ Smalltalk gtExamplesContained select: [ :each | 
		(each hasSubject: self)
			or: [ subjects anySatisfy: [ :subject | each hasSubject: subject ] ] ]! !

!GTExampleSubclassesSubject methodsFor: 'converting' stamp: 'StefanReichhart 7/22/2016 12:36'!
gtExamplesSubjects
	^ super gtExamplesSubjects , (self theSubclasses collect: [ :each | each asGTExampleSubject ])! !

!GTExampleSubclassesSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/22/2016 12:32'!
hasLiteralThorough: aLiteral
	^ (super hasLiteralThorough: aLiteral)
		or: [ self theSubclasses anySatisfy: [ :class | 
				class asGTExampleSubject hasLiteralThorough: aLiteral ] ]! !

!GTExampleSubclassesSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/22/2016 12:32'!
theSubclasses
	^ self exists 
		ifFalse: [ {  } ]
		ifTrue: [ self theClass allSubclasses ]! !

GTExampleSubject subclass: #GTExampleHelpSubject
	instanceVariableNames: 'bookName title topic'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleHelpSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:59'!
fromBook: aString
	^ self fromBook: aString title: nil! !

!GTExampleHelpSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:55'!
fromBook: aString1 title: aString2
	^ self new 
		bookName: aString1;
		title: aString2;
		yourself! !

!GTExampleHelpSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:58'!
fromTopic: aHelpTopic
	^ self fromBook: aHelpTopic title title: (aHelpTopic subtopics 
		ifNotEmpty: [ :topics | topics first title ]
		ifEmpty: [ nil ])! !

!GTExampleHelpSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self bookName = anExampleSubject bookName 
			and: [ self title = anExampleSubject title ] ]! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 14:22'!
bookName
	^ bookName! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 14:22'!
bookName: anObject
	bookName := anObject! !

!GTExampleHelpSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 14:08'!
exists
	^ self topic notNil! !

!GTExampleHelpSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	aBlock 
		value: #'Browse subject Help ' ,  self topic gtDisplayString , (self title ifNotNil: [ ' > ', self title ] ifNil: [ '' ])
		value: [ 
			| browser |
			browser := (HelpBrowser openOn: self topic).
			self title ifNotNil: [ browser selectTopicSatisfying: [ :subhelp | subhelp title = self title ] ] ]! !

!GTExampleHelpSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 15:56'!
gtDisplayOn: aStream
	self exists 
		ifTrue: [ self topic gtDisplayOn: aStream ]
		ifFalse: [ 
			self bookName gtDisplayOn: aStream.
			self title isEmptyOrNil ifFalse: [ 
				aStream nextPutAll: ' > '.
				self title gtDisplayOn: aStream ] ].
	self gtDisplayValidityOn: aStream! !

!GTExampleHelpSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 14:21'!
hasLiteralThorough: aLiteral
	^ self topic notNil 
		and: [ self topic contents notNil
			and: [ self topic contents includesSubstring: aLiteral asString caseSensitive: false ] ]! !

!GTExampleHelpSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
hash
	^ (self class hash
		bitXor: self bookName hash)
			bitXor: self title hash! !

!GTExampleHelpSubject methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub
	topic := nil.! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 14:06'!
title
	^ title! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 14:06'!
title: anObject
	title := anObject! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 17:28'!
topic
	topic ifNil: [ 
		CustomHelp withAllSubclassesDo: [ :class | 
			class asHelpTopic ifNotNil: [ :help |
				(help title = bookName
						and: [ title isNil 
							or: [ help subtopics anySatisfy: [ :subhelp | 
									subhelp title = title ] ] ]) ifTrue: [ 
					topic := help ] ] ] ].
	^ topic! !

!GTExampleHelpSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 14:07'!
topic: anObject
	topic := anObject! !

GTExampleSubject subclass: #GTExampleInstanceSubject
	instanceVariableNames: 'instance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleInstanceSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:00'!
fromObject: anObject
	^ self new 
		instance: anObject;
		yourself! !

!GTExampleInstanceSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self instance = anExampleSubject instance ]! !

!GTExampleInstanceSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 13:41'!
exists
	^ self theClass notNil! !

!GTExampleInstanceSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	aBlock
		value: #'Inspect subject Instance ' ,  self instance gtDisplayString
		value: [ self instance inspect ]! !

!GTExampleInstanceSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:10'!
gtDisplayOn: aStream
	self instance gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExampleInstanceSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 16:11'!
hasLiteralThorough: aLiteral
	^ self exists
		and: [ self theClass methods anySatisfy: [ :method | 
			method selector = aLiteral
				or: [ 	method hasLiteralThorough: aLiteral ] ] ]! !

!GTExampleInstanceSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
hash
	^ self class hash
		bitXor: self instance hash! !

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance
	^ instance! !

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance: anObject
	instance := anObject! !

!GTExampleInstanceSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/4/2016 13:41'!
theClass
	^ self instance class! !

GTExampleSubject subclass: #GTExampleKeywordSubject
	instanceVariableNames: 'keyword'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleKeywordSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:45'!
fromString: aString
	^ self new 
		keyword: aString;
		yourself! !

!GTExampleKeywordSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:47'!
fromSymbol: aSymbol
	^ self new 
		keyword: aSymbol;
		yourself! !

!GTExampleKeywordSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:47'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self keyword = anExampleSubject keyword ]! !

!GTExampleKeywordSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:08'!
exists
	^ true! !

!GTExampleKeywordSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 12:19'!
gtDisplayOn: aStream
	self keyword gtDisplayOn: aStream! !

!GTExampleKeywordSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:19'!
hasLiteralThorough: aLiteral
	^ self keyword = aLiteral
		or: [ self keyword asString asLowercase = aLiteral asString asLowercase ]! !

!GTExampleKeywordSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:47'!
hash
	^ self class hash
		bitXor: self keyword hash! !

!GTExampleKeywordSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 12:18'!
keyword
	^ keyword! !

!GTExampleKeywordSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 12:18'!
keyword: anObject
	keyword := anObject! !

GTExampleSubject subclass: #GTExamplePackageSubject
	instanceVariableNames: 'thePackageName thePackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExamplePackageSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:02'!
fromPackage: aRPackage
	^ self new 
		thePackage: aRPackage;
		thePackageName: aRPackage packageName;
		yourself! !

!GTExamplePackageSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:02'!
fromPackageNamed: aString
	^ self new 
		thePackage: nil;
		thePackageName: aString asString;
		yourself! !

!GTExamplePackageSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self thePackageName = anExampleSubject thePackageName ]! !

!GTExamplePackageSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:09'!
addToSelectorEnvironment: aSelectorEnvironment
	self thePackage ifNil: [ ^ self ].
	
	self thePackage classes do: [ :class | 
		aSelectorEnvironment addClass: class ].
	self thePackage extensionMethods do: [ :method | 
		aSelectorEnvironment addClass: method methodClass selector: method selector ]! !

!GTExamplePackageSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 13:56'!
exists
	^ self thePackage notNil! !

!GTExamplePackageSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:51'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock 
		value: #'Browse subject Package ' ,  self thePackageName
		value: [ self thePackage browse ]! !

!GTExamplePackageSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:10'!
gtDisplayOn: aStream
	self thePackageName gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExamplePackageSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/15/2016 19:59'!
hasLiteralThorough: aLiteral
	^ self thePackageName = aLiteral
		or: [ self exists
			and: [ (self thePackage classes anySatisfy: [ :each | each name = aLiteral asString ])
				or: [ self thePackage methods anySatisfy: [ :each | each hasLiteralThorough: aLiteral ] ] ] ]! !

!GTExamplePackageSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 18:39'!
hash
	^ self class hash
		bitXor: self thePackageName hash! !

!GTExamplePackageSubject methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub
	thePackage := nil! !

!GTExamplePackageSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 17:30'!
thePackage
	^ thePackage ifNil: [ thePackage := RPackageOrganizer default packages 
		detect: [ :each | each packageName = thePackageName ]
		ifNone: [ nil ] ]! !

!GTExamplePackageSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/22/2016 12:42'!
thePackage: anObject
	anObject ifNotNil: [ self thePackageName: anObject packageName ].
	thePackage := anObject! !

!GTExamplePackageSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:50'!
thePackageName
	^ thePackageName! !

!GTExamplePackageSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:50'!
thePackageName: anObject
	thePackageName := anObject! !

GTExampleSubject subclass: #GTExampleSelectorSubject
	instanceVariableNames: 'selector theClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleSelectorSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:42'!
fromString: aString
	^ self fromSymbol: aString asSymbol! !

!GTExampleSelectorSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:42'!
fromSymbol: aSymbol
	^ self new
		selector: aSymbol;
		yourself! !

!GTExampleSelectorSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:46'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self selector = anExampleSubject selector ]! !

!GTExampleSelectorSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 12:44'!
addToSelectorEnvironment: aSelectorEnvironment
	self selector ifNil: [ ^ self ].
	
	self theClasses do: [ :class | 
		aSelectorEnvironment addClass: class selector: self selector ]! !

!GTExampleSelectorSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:41'!
exists
	^ self theClasses isEmptyOrNil not! !

!GTExampleSelectorSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 12:45'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock 
		value: #'Browse subject Selector ' ,  self selector
		value: [ 
			| env |
			self addToSelectorEnvironment: (env := RBSelectorEnvironment new).
			env browse ]! !

!GTExampleSelectorSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 12:43'!
gtDisplayOn: aStream
	self selector gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExampleSelectorSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:42'!
hasLiteralThorough: aLiteral
	^ self selector = aLiteral
		or: [ self theClasses anySatisfy: [ :class |
			(class >> self selector) asGTExampleSubject hasLiteralThorough: aLiteral ] ]! !

!GTExampleSelectorSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:47'!
hash
	^ self class hash
		bitXor: self selector hash! !

!GTExampleSelectorSubject methodsFor: 'private' stamp: 'StefanReichhart 8/13/2016 12:38'!
resetStub
	theClasses := nil! !

!GTExampleSelectorSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 12:35'!
selector
	^ selector! !

!GTExampleSelectorSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 12:35'!
selector: anObject
	selector := anObject! !

!GTExampleSelectorSubject methodsFor: 'private' stamp: 'StefanReichhart 8/13/2016 12:40'!
theClasses
	theClasses ifNil: [ 
		theClasses := Array streamContents: [ :stream |
			Smalltalk allClassesAndTraitsDo: [ :class |
				(class includesSelector: self selector) ifTrue: [ 
					stream nextPut: class ] ] ] ].
	^ theClasses! !

!GTExampleSubject class methodsFor: 'testing' stamp: 'TudorGirba 8/21/2016 07:43'!
isAbstract
	^ self == GTExampleSubject! !

!GTExampleSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
= anExampleSubject
	^ self subclassResponsibility! !

!GTExampleSubject methodsFor: 'converting' stamp: 'StefanReichhart 7/2/2016 19:45'!
asGTExampleSubject
	^ self! !

!GTExampleSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:12'!
gtDisplayOn: aStream
	super gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExampleSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/22/2016 12:02'!
gtExamplesContained
	^ Smalltalk gtExamplesContained select: [ :each | each hasSubject: self ]! !

!GTExampleSubject methodsFor: 'converting' stamp: 'StefanReichhart 7/22/2016 12:35'!
gtExamplesSubjects
	^ { self }! !

!GTExampleSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/4/2016 20:58'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 102>
	aStep listProcessor
		title: 'Examples with Subject ', self gtDisplayString;
		filter: GTNullFilter item: [ :filter :context | 
			GTExampleOrganizer instance examplesDo: [ :cn :s :example |
				(example hasSubject: self) ifTrue: [ filter value: example ] ] ];
		wantsToDisplayOnEmptyQuery: true! !

!GTExampleSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
hash
	^ self subclassResponsibility! !

GTExampleSubject subclass: #GTExampleTagSubject
	instanceVariableNames: 'theTagName theTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleTagSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:03'!
fromTag: aRPackageTag
	^ self new 
		theTag: aRPackageTag;
		theTagName: aRPackageTag categoryName;
		yourself! !

!GTExampleTagSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 16:04'!
fromTagNamed: aString
	^ self new 
		theTag: nil;
		theTagName: aString asString;
		yourself! !

!GTExampleTagSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self theTagName = anExampleSubject theTagName ]! !

!GTExampleTagSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:07'!
addToSelectorEnvironment: aSelectorEnvironment
	self thePackage ifNil: [ ^ self ].
	
	self thePackage classes do: [ :class | 
		aSelectorEnvironment addClass: class ]! !

!GTExampleTagSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 14:45'!
exists
	^ self theTag notNil! !

!GTExampleTagSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/11/2016 16:50'!
forNautilus: aBlock
	self exists ifFalse: [ ^ self ].
	
	aBlock 
		value: #'Browse subject Tag ' ,  self theTagName
		value: [ self theTag browse ]! !

!GTExampleTagSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:10'!
gtDisplayOn: aStream
	self theTagName gtDisplayOn: aStream.
	self gtDisplayValidityOn: aStream! !

!GTExampleTagSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/15/2016 19:59'!
hasLiteralThorough: aLiteral
	^ self theTagName = aLiteral
		or: [ self exists
			and: [ self theTag categoryName = aLiteral
				or: [ (self theTag classes anySatisfy: [ :each | each name = aLiteral asString ])
					"or: [ self theTag methods anySatisfy: [ :each | each hasLiteralThorough: aLiteral ] ]" ] ] ]! !

!GTExampleTagSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
hash
	^ self class hash
		bitXor: self theTagName hash! !

!GTExampleTagSubject methodsFor: 'private' stamp: 'StefanReichhart 7/21/2016 17:34'!
resetStub
	theTag := nil! !

!GTExampleTagSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/11/2016 16:44'!
thePackage
	^ self theTag package! !

!GTExampleTagSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 17:31'!
theTag
	theTag ifNil: [ 
		RPackageOrganizer default packages do: [ :package |
			package classTags do: [ :tag | 
				(tag name = theTagName
					or: [ tag categoryName = theTagName ]) ifTrue: [ 
						^ theTag := tag ] ] ] ].
	^ theTag! !

!GTExampleTagSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/22/2016 12:43'!
theTag: anObject
	anObject ifNotNil: [ self theTagName: anObject name ].
	theTag := anObject! !

!GTExampleTagSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:59'!
theTagName
	^ theTagName! !

!GTExampleTagSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/4/2016 13:59'!
theTagName: anObject
	theTagName := anObject! !

GTExampleSubject subclass: #GTExampleTextSubject
	instanceVariableNames: 'text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Subjects'!

!GTExampleTextSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:43'!
fromString: aString
	^ self new 
		text: aString;
		yourself! !

!GTExampleTextSubject class methodsFor: 'instance creation' stamp: 'StefanReichhart 8/13/2016 15:43'!
fromText: aText
	^ self fromString: aText asString! !

!GTExampleTextSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
= anExampleSubject
	^ self class = anExampleSubject class
		and: [ self text = anExampleSubject text ]! !

!GTExampleTextSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 11:54'!
exists
	^ true! !

!GTExampleTextSubject methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 8/13/2016 11:56'!
gtDisplayOn: aStream
	self text gtDisplayOn: aStream! !

!GTExampleTextSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 11:55'!
hasLiteralThorough: aLiteral
	^ self text = aLiteral
		or: [ self text includesSubstring: aLiteral asString caseSensitive: false ]! !

!GTExampleTextSubject methodsFor: 'testing' stamp: 'StefanReichhart 8/13/2016 12:48'!
hash
	^ self class hash
		bitXor: self text hash! !

!GTExampleTextSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 11:53'!
text
	^ text! !

!GTExampleTextSubject methodsFor: 'accessing' stamp: 'StefanReichhart 8/13/2016 11:53'!
text: anObject
	text := anObject! !

Object subclass: #GTExampleFactory
	instanceVariableNames: 'cache source provider exampleClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
cache
	^ cache ifNil: [ cache := self defaultCache ]! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
cache: anObject
	^ cache := anObject! !

!GTExampleFactory methodsFor: 'private-utils' stamp: 'TudorGirba 8/20/2016 00:40'!
createExample: aCompiledMethod
	^ self exampleClass new
		initializeFromMethod: aCompiledMethod usingFactory: self;
		yourself! !

!GTExampleFactory methodsFor: 'accessing-defaults' stamp: 'TudorGirba 8/20/2016 00:39'!
defaultCache
	^ GTExampleOrganizer instance! !

!GTExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 19:16'!
defaultExampleClass
	^ GTExample! !

!GTExampleFactory methodsFor: 'accessing-defaults' stamp: 'TudorGirba 8/20/2016 00:39'!
defaultProviderFor: aCompiledMethod 
	^ aCompiledMethod methodClass isMeta
		ifTrue: [ aCompiledMethod methodClass theNonMetaClass ]
		ifFalse: [ aCompiledMethod methodClass theNonMetaClass new " WARNING: not all classes support this -> override this using: #provider: " ] ! !

!GTExampleFactory methodsFor: 'private' stamp: 'TudorGirba 8/20/2016 00:40'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ aBlock value! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
exampleClass
	^ exampleClass ifNil: [ exampleClass := self defaultExampleClass ]! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
exampleClass: anObject
	exampleClass := anObject! !

!GTExampleFactory methodsFor: 'private' stamp: 'TudorGirba 8/20/2016 00:40'!
exampleFromMethod: aCompiledMethod do: aBlock
	aBlock value: (self cache exampleAt: aCompiledMethod ifAbsentPut: [ self createExample: aCompiledMethod ])! !

!GTExampleFactory methodsFor: 'private' stamp: 'TudorGirba 8/20/2016 00:40'!
exampleMethod: aCompiledMethod do: aBlock
	(self isExampleMethod: aCompiledMethod) ifFalse: [ ^ self ].
	self exampleFromMethod: aCompiledMethod do: aBlock! !

!GTExampleFactory methodsFor: 'private' stamp: 'TudorGirba 8/20/2016 00:40'!
exampleMethodsDo: aBlock
	self source theNonMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
	self source theMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
		! !

!GTExampleFactory methodsFor: 'private' stamp: 'TudorGirba 8/20/2016 00:40'!
examplesDo: aBlock
	self exampleMethodsDo: [ :method |
		self exampleFromMethod: method do: aBlock ]! !

!GTExampleFactory methodsFor: 'public' stamp: 'TudorGirba 8/20/2016 00:40'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self examplesDo: [ :example | examples add: example ].
	^ examples! !

!GTExampleFactory methodsFor: 'private - creating examples' stamp: 'TudorGirba 8/21/2016 07:12'!
initializeExample: aGTExample fromMethod: aCompiledMethod
	self initializeSubjectsForMethod: aCompiledMethod forExample: aGTExample.
	self initializePragmasFromMethod: aCompiledMethod forExample: aGTExample.
! !

!GTExampleFactory methodsFor: 'private-init' stamp: 'TudorGirba 8/20/2016 00:40'!
initializePragmas: aCollection forExample: aGTExample
	aCollection do: [ :pragma | 
		(aGTExample class canPerform: pragma keyword) ifTrue: [ 
			[ aGTExample perform: pragma keyword withArguments: pragma arguments ]
				on:Error
				do:[ :exception | 
					aGTExample addProblem: (GTExamplePragmaError new 
						example: aGTExample;
						pragma: pragma;
						signaledException: exception;
						yourself) ] ] ]! !

!GTExampleFactory methodsFor: 'private-init' stamp: 'TudorGirba 8/20/2016 00:40'!
initializePragmasFromMethod: aCompiledMethod forExample: aGTExample
	self initializePragmas: aCompiledMethod pragmas forExample: aGTExample! !

!GTExampleFactory methodsFor: 'private-init' stamp: 'TudorGirba 8/20/2016 00:40'!
initializeSubjects: aCollection forExample: aGTExample
	aCollection isEmptyOrNil ifTrue: [ ^ self ].
	
	aCollection do: [ :subject | 
		[ aGTExample addSubject: subject ]
			on:Error
			do: [ :exception | 
				aGTExample addProblem: (GTExampleSubjectError new 
					example: aGTExample;
					subject: subject;
					signaledException: exception;
					yourself) ] ] ! !

!GTExampleFactory methodsFor: 'private-init' stamp: 'TudorGirba 8/20/2016 00:40'!
initializeSubjectsForMethod: aCompiledMethod forExample: aGTExample
	[ self providerFor: aCompiledMethod do: [ :each | 
			(each class canPerform: #gtExamplesSubjects) ifTrue: [
				self initializeSubjects: each gtExamplesSubjects forExample: aGTExample ] ] ]
		on: Error
		do: [ :exception | 
			aGTExample addProblem: (GTExampleSubjectError new 
				example: aGTExample;
				signaledException: exception;
				yourself) ]! !

!GTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:00'!
isExampleMethod: aCompiledMethod
	^ self isGTExampleMethod: aCompiledMethod ! !

!GTExampleFactory methodsFor: 'testing' stamp: 'TudorGirba 8/20/2016 00:40'!
isGTExampleMethod: aCompiledMethod
	^ aCompiledMethod isGTExampleMethod! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
provider
	^ provider! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
provider: anObject
	provider := anObject! !

!GTExampleFactory methodsFor: 'private-utils' stamp: 'TudorGirba 8/20/2016 00:40'!
providerFor: aCompiledMethod
	^ provider ifNil: [ self defaultProviderFor: aCompiledMethod ]! !

!GTExampleFactory methodsFor: 'private-utils' stamp: 'TudorGirba 8/20/2016 00:40'!
providerFor: aCompiledMethod do: aBlock1
	^ self providerFor: aCompiledMethod do: aBlock1 ifNone: nil! !

!GTExampleFactory methodsFor: 'private-utils' stamp: 'TudorGirba 8/20/2016 00:40'!
providerFor: aCompiledMethod do: aBlock1 ifNone: aBlock2
	^ (self providerFor: aCompiledMethod)
		ifNotNil: aBlock1
		ifNil: aBlock2! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
source
	^ source! !

!GTExampleFactory methodsFor: 'accessing' stamp: 'TudorGirba 8/20/2016 00:39'!
source: anObject
	source := anObject! !

Object subclass: #GTExampleOrganizer
	instanceVariableNames: 'theNonMetaClassExamples theMetaClassExamples results systemAnnouncements monticelloAnnouncements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Organization'!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

!GTExampleOrganizer commentStamp: 'TudorGirba 8/21/2016 07:19' prior: 0!
I am a singleton that keeps track of all examples defined in the image.

I know how to traverse the image to iterate over examples.!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
instance
	^ instance ifNil: [ instance := self basicNew initialize start ]! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 9/7/2015 21:42'!
new
	self shouldNotImplement! !

!GTExampleOrganizer class methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 15:00'!
registerInterestToSystemAnnouncement
	<systemEventRegistration>
	
	self restart! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 20:58'!
reset
	instance ifNotNil: [ instance reset ].
	instance := nil! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
restart
	self stop.
	self start.! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
start
	self instance start! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:57'!
stop
	instance ifNotNil: [ instance stopThoroughly ].
	self reset! !

!GTExampleOrganizer methodsFor: 'private-maintenance' stamp: 'StefanReichhart 7/4/2016 17:06'!
addExamplesOfClass: aClassName for: announcement
	(Smalltalk classNamed: aClassName asString) ifNotNil: [ :class | 
		announcement addExamples: class theNonMetaClass gtExamples ]! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:44'!
announcer
	^ SystemAnnouncer uniqueInstance weak! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 23:58'!
browserEnvironment
	| environment |
	environment := RBSelectorEnvironment new.
	self examplesDo: [ :className :selector :example |
		environment addClass: example methodClass selector: example selector ].
	^ environment! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/11/2016 15:49'!
categoryRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self examplesDo: [ :example | ann removeExample: example ].
			self reset.
			self examplesDo: [ :example | ann addExample: example ]. ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/21/2016 17:34'!
classRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self resetStubs.
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self removeExamplesOfClass: anAnnouncement classRemoved theNonMetaClass name for: ann ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/21/2016 17:34'!
classRenamed: anAnnouncement
	self withoutExceptionsDo: [ 
		self resetStubs.
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self removeExamplesOfClass: anAnnouncement oldName for: ann.
			self addExamplesOfClass: anAnnouncement newName for: ann ] ]! !

!GTExampleOrganizer methodsFor: 'TODO' stamp: 'StefanReichhart 7/5/2016 16:22'!
exampleAt: aCompiledMethod
	^ self exampleAt: aCompiledMethod ifAbsent: [ aCompiledMethod gtExample ]! !

!GTExampleOrganizer methodsFor: 'TODO' stamp: 'StefanReichhart 7/5/2016 16:23'!
exampleAt: aCompiledMethod ifAbsent: aBlock
	^ self exampleAtClass: aCompiledMethod methodClass selector: aCompiledMethod selector ifAbsent: aBlock! !

!GTExampleOrganizer methodsFor: 'TODO' stamp: 'StefanReichhart 7/2/2016 21:04'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ self exampleAtClass: aCompiledMethod methodClass selector: aCompiledMethod selector ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/4/2016 20:07'!
exampleAtClass: class selector: selector ifAbsent: aBlock
	^ ((class isMeta 
		ifTrue: [ theMetaClassExamples ]
		ifFalse: [ theNonMetaClassExamples ]) 
			at: class theNonMetaClass name
			ifAbsentPut: [ WeakIdentityKeyDictionary new: 100 ])
				at: selector 
				ifAbsent: aBlock! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 16:21'!
exampleAtClass: class selector: selector ifAbsentPut: aBlock
	| methodDict |
	methodDict := (class isMeta 
		ifTrue: [ theMetaClassExamples ]
		ifFalse: [ theNonMetaClassExamples ]) 
			at: class theNonMetaClass name
			ifAbsentPut: [ WeakIdentityKeyDictionary new: 100 ].
	^ methodDict 
		at: selector 
		ifAbsent: [ aBlock value ifNotNil: [ :example | methodDict at: selector put: example ] ]! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/5/2016 13:29'!
examplesCollect: aBlock
	| examples |
	examples := OrderedCollection new: 1000.
	self examplesDo: [ :class :method :example | 
		examples add: (aBlock value: example) ].
	^ examples! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:01'!
examplesDetect: aBlock
	self examplesDo: [ :class :selector :example | 
		(aBlock value: example) ifTrue: [ ^ example ] ].
	^ nil! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:00'!
examplesDo: aBlock
	theNonMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ].
	theMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ]! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:02'!
examplesReject: aBlock
	^ self examplesSelect: [ :example | (aBlock value: example) not ]! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/5/2016 13:29'!
examplesSelect: aBlock
	| examples |
	examples := OrderedCollection new: 1000.
	self examplesDo: [ :class :selector :example | 
		(aBlock value: example) ifTrue: [ examples add: example ] ].
	^ examples! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/5/2016 13:29'!
examplesSelect: aBlock1 thenDo: aBlock2
	| examples |
	examples := OrderedCollection new: 1000.
	self examplesDo: [ :example |
		(aBlock2 value: example) ifTrue: [ 
			examples add: example.
			aBlock1 value: example ] ].
	^ examples! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 23:46'!
gtExamplesContained
	^ self examplesCollect: [ :each | each ]! !

!GTExampleOrganizer methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/4/2016 20:58'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		filter: GTNullFilter item: [ :filter :context | 
			self examplesDo: [ :cn :s :example | filter value: example ] ]! !

!GTExampleOrganizer methodsFor: 'private-maintenance' stamp: 'StefanReichhart 7/4/2016 19:02'!
handleGTReservedMethod: aCompiledMethod for: announcement
	aCompiledMethod isGTReservedMethod ifFalse: [ ^ self ].
	self removeExamplesOfClass: aCompiledMethod methodClass theNonMetaClass name for: announcement! !

!GTExampleOrganizer methodsFor: 'initializing' stamp: 'StefanReichhart 7/5/2016 15:25'!
initialize
	super initialize.
	
	theNonMetaClassExamples := WeakIdentityKeyDictionary new: 10000.
	theMetaClassExamples := WeakIdentityKeyDictionary new: 10000.
	results := LRUCache new
		maximumWeight: 20;
		yourself! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/4/2016 17:09'!
loadStarted: anAnnouncement
	self withoutExceptionsDo: [
		self stopSystemAnnouncements.
		self reset ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/4/2016 17:09'!
loadStopped: anAnnouncement
	self withoutExceptionsDo: [ 
		self startSystemAnnouncements.
		self reinitialize ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/11/2016 15:49'!
methodAdded: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self handleGTReservedMethod: anAnnouncement method for: ann.
			self removeExamplesOfMethod: anAnnouncement method for: ann.
			ann addExample: anAnnouncement method gtExample ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/21/2016 17:34'!
methodModified: anAnnouncement
	self withoutExceptionsDo: [ 
		self resetStubs.
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self handleGTReservedMethod: anAnnouncement oldMethod for: ann.
			self handleGTReservedMethod: anAnnouncement method for: ann.
			self removeExamplesOfMethod: anAnnouncement oldMethod for: ann.
			self removeExamplesOfMethod: anAnnouncement method for: ann.
			ann addExample: anAnnouncement method gtExample ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/21/2016 17:34'!
methodRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self resetStubs.
		self with: GTExamplesModifiedAnnouncement announcement: anAnnouncement do: [ :ann |
			self handleGTReservedMethod: anAnnouncement method for: ann.
			self removeExamplesOfMethod: anAnnouncement method for: ann ] ]! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:55'!
reinitialize
	SmalltalkImage current gtExamplesContained! !

!GTExampleOrganizer methodsFor: 'private-maintenance' stamp: 'StefanReichhart 7/4/2016 17:03'!
removeExamplesOfClass: aClassName for: announcement
	announcement removeExamples: (theNonMetaClassExamples removeKey: aClassName ifAbsent: [ nil ]).
	announcement removeExamples: (theMetaClassExamples removeKey: aClassName	ifAbsent: [ nil ])! !

!GTExampleOrganizer methodsFor: 'private-maintenance' stamp: 'StefanReichhart 7/14/2016 16:17'!
removeExamplesOfMethod: aCompiledMethod for: announcement
	self removeKey: aCompiledMethod methodClass theNonMetaClass name from: theNonMetaClassExamples thenDo: [ :methodDict |
		announcement removeExample: (methodDict removeKey: aCompiledMethod selector ifAbsent: [ nil ]) ].
	self removeKey: aCompiledMethod methodClass theNonMetaClass name from: theMetaClassExamples thenDo: [ :methodDict |
		announcement removeExample: (methodDict removeKey: aCompiledMethod selector ifAbsent: [ nil ]) ]! !

!GTExampleOrganizer methodsFor: 'private-maintenance' stamp: 'StefanReichhart 7/14/2016 16:17'!
removeKey: aKey from: aDictionary thenDo: aBlock
	aBlock value: (aDictionary removeKey: aKey ifAbsent: [ ^ nil ])! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:02'!
reset
	self initialize! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'TudorGirba 8/20/2016 00:29'!
resetStubs
	[ GTExampleAspect allSubInstancesDo: [ :each | each resetStub ] ] 
		forkAt: 30
		named: 'GTExampleOrganizer >> #resetStubs'! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:55'!
restart
	self stopThoroughly.
	self start.! !

!GTExampleOrganizer methodsFor: 'TODO' stamp: 'StefanReichhart 7/5/2016 16:01'!
resultAt: aGTExample ifAbsent: aBlock
	^ results at: { aGTExample methodClass. aGTExample selector } ifAbsent: aBlock! !

!GTExampleOrganizer methodsFor: 'TODO' stamp: 'StefanReichhart 7/13/2016 23:05'!
resultAt: aGTExample put: aGTExampleResult
	GTAbstractExampleNautilusActionButton methodUpdated: aGTExample method with: aGTExampleResult.
	^ results at: { aGTExample methodClass. aGTExample selector } put: aGTExampleResult! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
start
	self startMonticelloAnnouncements.
	self startSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 11:48'!
startMonticelloAnnouncements
	monticelloAnnouncements == true ifTrue: [ ^ self ].
	SystemAnnouncer uniqueInstance weak
		when: MCVersionLoaderStarted send: #loadStarted: to: self;
		when: MCVersionLoaderStopped send: #loadStopped: to: self.
	monticelloAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 13:14'!
startSystemAnnouncements
	systemAnnouncements == true ifTrue: [ ^ self ].
	SystemAnnouncer uniqueInstance weak
		when: MethodAdded send: #methodAdded: to: self;
		when: MethodModified send: #methodModified: to: self;
		when: MethodRemoved send: #methodRemoved: to: self;
		when: ClassRemoved send: #classRemoved: to: self;
		when: ClassRenamed send: #classRenamed: to: self;
		when: CategoryRemoved send: #categoryRemoved: to: self.
	systemAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stop
	self stopMonticelloAnnouncements.
	self stopSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 13:14'!
stopMonticelloAnnouncements
	SystemAnnouncer uniqueInstance
		unsubscribe: MCVersionLoaderStarted thoroughlyFor: self;
		unsubscribe: MCVersionLoaderStopped thoroughlyFor: self.
	monticelloAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 13:14'!
stopSystemAnnouncements
	SystemAnnouncer uniqueInstance
		unsubscribe: MethodAdded thoroughlyFor: self;
		unsubscribe: MethodModified thoroughlyFor: self;
		unsubscribe: MethodRemoved thoroughlyFor: self;
		unsubscribe: ClassRemoved thoroughlyFor: self;
		unsubscribe: ClassRenamed thoroughlyFor: self;
		unsubscribe: CategoryRemoved thoroughlyFor: self.
	systemAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stopThoroughly
	self stop.
	
	Announcer allSubInstances do: [ :each | each unsubscribeThoroughly: self ]! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/5/2016 11:49'!
with: aClass announcement: announcement do: aBlock
	| wrappedAnnouncement |
	wrappedAnnouncement := aClass new
		announcement: announcement;
		yourself.
	aBlock value: wrappedAnnouncement.
	wrappedAnnouncement isEmpty ifTrue: [ ^ self ].
	SystemAnnouncer uniqueInstance announce: wrappedAnnouncement! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:14'!
withoutExceptionsDo: aBlock
	[ aBlock value ]
		on: Error
		do: [ :exception | " we should not ignore it, but try to continue " ]! !

Object subclass: #GTExampleProcessor
	instanceVariableNames: 'example context processing onProcess onRecursionError onDependencyError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTExampleProcessor commentStamp: 'TudorGirba 8/21/2016 14:44' prior: 0!
I am an abstract class for manipulating examples.

I know how to !

GTExampleProcessor subclass: #GTExampleEvaluator
	instanceVariableNames: 'processed result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

GTExampleEvaluator subclass: #GTExampleDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTExampleDebugger methodsFor: 'private' stamp: 'StefanReichhart 7/13/2015 12:14'!
do: aBlock on: anException do: anotherBlock
	aBlock value! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'TudorGirba 8/21/2016 08:02'!
do: aBlock on: anException do: anotherBlock
	"I give a chance to my subclass to not handle the exception is needed"
	aBlock
		on: anException
		do: anotherBlock! !

!GTExampleEvaluator methodsFor: 'initializing' stamp: 'StefanReichhart 5/13/2016 11:04'!
initialize
	super initialize.
	
	processed := OrderedCollection new! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 5/13/2016 11:38'!
newResultFor: anExample
	^ GTExampleResult new 
		example: anExample;
		context: self context;
		yourself.! !

!GTExampleEvaluator methodsFor: 'primitives' stamp: 'StefanReichhart 5/13/2016 10:31'!
primitiveProcessExample: anExample withArguments: arguments
	^ anExample provider perform: anExample method selector withArguments: arguments! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'TudorGirba 8/21/2016 14:47'!
process: anExample withArguments: aBlock
	| currentResult arguments |
	currentResult := self newResultFor: anExample.
	arguments := nil.
	anExample hasValidDependencies " dynamic check "
		ifFalse: [ ^ self handleDependencyErrorFor: anExample ].
	self 
		do: [ 
			arguments := aBlock value. 
			currentResult returnValue: (self 
				primitiveProcessExample: anExample 
				withArguments: arguments) ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	processed add: currentResult.
	self
		do: [ self processAfter: anExample ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	anExample == self example ifTrue: [ result := currentResult ].
	anExample exceptions signaledBy: currentResult expectedError.
	^ currentResult returnValue! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 6/24/2016 11:52'!
processAfter: anExample
	anExample == self example ifFalse: [ ^ self ].
	processed reverseDo: [ :processedResult |
		self processAfter: processedResult example withResult: processedResult returnValue ]! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 8/14/2016 13:12'!
processAfter: anExample withResult: anObject
	anExample after ifNil: [ ^ self ].
	anExample after method
		gtValueWithReceiver: anExample after provider 
		possibleArguments: { anObject }! !

!GTExampleEvaluator methodsFor: 'public' stamp: 'StefanReichhart 7/5/2016 16:08'!
result
	result := self newResultFor: self example.
	self do: [ self value ] 
		on: Error
		do: [ :exception | result returnValue: nil; unexpectedError: exception ].
	GTExampleOrganizer instance resultAt: self example put: result.
	^ result! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/14/2016 18:01'!
basicProcess: anExample 
	^ self 
		process: anExample 
		withArguments: [ anExample dependenciesOrArguments collect: [ :each | self process: each ] ] ! !

!GTExampleProcessor methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:44'!
canProcess
	[ self value ]
		on: GTExampleError
		do: [ ^ false ].
	^ true! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context
	^ context! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context: anObject
	^ context := anObject! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 9/2/2015 20:58'!
defaultOnProcess
	^ [ :aProcessor :anExample | 
		aProcessor basicProcess: anExample ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example
	^ example! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example: anObject
	example := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/14/2016 18:23'!
handleDependencyErrorFor: anExample
	^ (GTExampleDependencyError new
		example: anExample;
		defined: anExample arguments;
		expected: anExample methodArgumentNames;
		invalid: anExample invalidArguments;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:15'!
handleRecursionErrorFor: anExample
	^ (GTExampleRecursionDetected new 
		example: self example;
		resolved: processing;
		next: anExample;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'initializing' stamp: 'StefanReichhart 3/28/2016 20:17'!
initialize
	super initialize.
	
	processing := IdentitySet new.
	context := GTExampleContext new! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess
	^ onProcess ifNil: [ onProcess := self defaultOnProcess ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess: anObject
	onProcess := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 8/23/2015 15:23'!
process: anExample
	^ self process: anExample withoutLoopsDo: [ 
		self onProcess value: self value: anExample ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 16:55'!
process: anExample withArguments: aBlock
	self subclassResponsibility! !

!GTExampleProcessor methodsFor: 'private' stamp: 'TudorGirba 8/21/2016 14:41'!
process: anExample withoutLoopsDo: aBlock
	| value |
	" check whether we already scheduled the given example for processing -> prevent recursion "
	value := nil.
	processing do: [ :processingExample |
		(processingExample = anExample) ifTrue: [
			^ self handleRecursionErrorFor: anExample ] ].	
	processing add: anExample. " prepare processing this example "
	context example: anExample.
	value := aBlock value. " process this example "
	processing remove: anExample. " done processing "
	^ value! !

!GTExampleProcessor methodsFor: 'public' stamp: 'StefanReichhart 3/28/2016 18:55'!
value 
	^ self withContextDo: [ self process: self example ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 3/28/2016 20:17'!
withContextDo: aBlock
	GTCurrentExampleContext use: context during: aBlock! !

GTExampleProcessor subclass: #GTExampleTraverser
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action
	^ action! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action: anObject
	action := anObject! !

!GTExampleTraverser methodsFor: 'private' stamp: 'TudorGirba 8/21/2016 08:00'!
process: anExample withArguments: aBlock
	anExample hasValidDependencies
		ifFalse: [ ^ self handleDependencyErrorFor: anExample ].
	self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ]! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 16:46'!
reversed
	self onProcess: [ :processor :anExample | 
		self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ].
		anExample directDependents do: [ :each | processor process: each ] ]! !

Object subclass: #GTExampleResult
	instanceVariableNames: 'example context returnValue expectedError unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTExampleResult commentStamp: 'TudorGirba 8/20/2016 00:08' prior: 0!
I represent the result of executing an example.

I keep track of the
- returnValue - the real object returned by the example method
- expectedErrors, unexpectedErrors - collections of possible errors encountered when executing the example method!

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context
	^ context! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context: anObject
	context := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
debug
	self isSuccess ifTrue: [ ^ self ].
	^ self example debug! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example
	^ example! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example: anObject
	example := anObject! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError
	^ expectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError: anObject
	expectedError := anObject! !

!GTExampleResult methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:16'!
gtDisplayOn: aStream
	self example gtDisplayOn: aStream. 
	
	self unexpectedError ifNotNil: [ 
		aStream nextPutAll: ' ['.
		self unexpectedError gtDisplayOn: aStream.
		aStream nextPutAll: ']' ]! !

!GTExampleResult methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/15/2016 20:17'!
gtDisplayValidityOn: aStream
	self isSuccess ifTrue: [ ^ self ].
	
	aStream nextPutAll: ' ['.
	self unexpectedError gtDisplayOn: aStream.
	aStream nextPutAll: ']'! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 22:54'!
hasProblems
	^ self example hasProblems
		or: [ self isFailure
			or: [ self isError ] ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:11'!
isError
	^ self isSuccess not
		and: [ self isUnexpectedError
			and: [ self unexpectedError isGTExampleError ] ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:53'!
isExpectedError
	^ self expectedError notNil! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:11'!
isFailure
	^ self isSuccess not
		and: [ self isUnexpectedError
			and: [ self unexpectedError isGTExampleError not ] ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 7/6/2016 13:30'!
isSuccess
	^ self unexpectedError isNil
		and: [ self example isValid ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:54'!
isUnexpectedError
	^ self unexpectedError notNil! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue
	^ returnValue! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue: anObject
	returnValue := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
run
	^ self example run! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
unexpectedError
	^ unexpectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:47'!
unexpectedError: anObject
	unexpectedError := anObject! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:53'!
gtExampleContext
	^ GTCurrentExampleContext value! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 16:00'!
asGTExampleSubject
	^ GTExampleInstanceSubject fromObject: self! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:52'!
gtExampleContext
	^ self class gtExampleContext! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 12:23'!
gtExampleSubstrings
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:32'!
gtExamples
	^ self class gtExamples! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:31'!
gtExamplesContained
	^ self gtExamples! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:50'!
gtExamplesFactory
	^ self class gtExamplesFactory! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:36'!
gtExamplesSubjects
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 21:58'!
isGTExample
	^ false! !

!Object methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 7/11/2016 13:08'!
isGTExampleError
	^ false! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:33'!
isGTExampleMethod
	^ false! !

!WeakSubscriptionBuilder methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:18'!
unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber
	announcer unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber! !

!WeakSubscriptionBuilder methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/5/2016 11:40'!
unsubscribeThoroughly: aSubscriber
	announcer unsubscribeThoroughly: aSubscriber! !

!SmalltalkImage class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/4/2016 19:44'!
gtExamplesFactory
	^ super gtExamplesFactory
		provider: SmalltalkImage current;
		yourself! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:20'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	Object withAllSubclassesDo: [ :class |
		class gtExamplesFactory examplesDo: [ :example | 
			examples add: example ] ].
	^ examples! !

Notification subclass: #GTCurrentExampleContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTCurrentExampleContext class methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:45'!
defaultValue
	^ nil! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
use: anObject during: aBlock
	^ aBlock 
		on: self 
		do: [ :notification | notification resume: anObject ]! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
value
	^ self signal! !

!GTCurrentExampleContext methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:44'!
defaultAction
	^ self class defaultValue! !

!String methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 15:43'!
asGTExampleSubject
	^ GTExampleTextSubject fromString: self! !

!String methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 12:22'!
gtExampleSubstrings
	^ ((self substrings: ' .,;') 
		collect: [ :each | each trimBoth ])
		reject: [ :each | each isEmptyOrNil ]! !

!RPackage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 16:02'!
asGTExampleSubject
	^ GTExamplePackageSubject fromPackage: self! !

!RPackage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/12/2016 11:12'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

!Context class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/6/2016 15:07'!
gtExamplesFactory
	^ super gtExamplesFactory
		provider: self;
		yourself! !

IdentityDictionary subclass: #GTExampleContext
	instanceVariableNames: 'example'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Runtime'!

!GTExampleContext methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 13:20'!
example
	^ example! !

!GTExampleContext methodsFor: 'accessing' stamp: 'StefanReichhart 7/11/2016 13:20'!
example: anObject
	example := anObject! !

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:18'!
unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! "
			 and: [ subscription announcementClass == anAnnouncementClass ] ] ]
! !

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 2/26/2016 10:41'!
unsubscribeThoroughly: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! " ] ]
! !

!Magnitude class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/6/2016 14:57'!
gtExamplesFactory
	^ super gtExamplesFactory
		provider: self;
		yourself! !

TestCase subclass: #GTExampleTestCase
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleTestCase commentStamp: 'TudorGirba 8/21/2016 07:41' prior: 0!
I am an abstract bridge between examples and SUnit.

Subclasses can specify the packages from which I should collect the examples and run them as tests.!

!GTExampleTestCase class methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 11:24'!
isAbstract
	^ self name = #GTExampleTestCase! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:12'!
package: package matchesPackagePattern: packageName
	^ package packageName = packageName
		or: [ (packageName match: package packageName) 
			or: [ [ package packageName matchesRegexIgnoringCase: packageName ] 
						on: Error 
						do: [ false ] ] ]! !

!GTExampleTestCase methodsFor: 'configuration' stamp: 'StefanReichhart 9/26/2015 10:17'!
packageNames
	^ {  }! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:11'!
packages: aCollection
	| packages |
	packages := OrderedCollection new.
	RPackageOrganizer default packages do: [ :package |
		aCollection do: [ :packageName |
			(self package: package matchesPackagePattern: packageName) ifTrue: [ 
				packages add: package ] ] ].
	^ packages! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:11'!
run: aResult
	result := aResult.
	[ super run: aResult ] ensure: [ result := nil ]! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:12'!
runCase
	result ifNil: [ result := self classForTestResult new " fake for debug runner " ].
	super runCase. ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:56'!
runExample: example
	" this is not the testing tool we have in mind, but a temporary solution to integrate examples into the currently existing testing tools "
	| exampleResult |
	example ignoreTest ifTrue: [ ^ self ].
	self shouldnt: [ exampleResult := example run ] raise: Error.
	exampleResult isError 
		ifTrue: [ ^ self classForTestResult signalErrorWith: exampleResult gtDisplayString  ].
	exampleResult isFailure 
		ifTrue: [ ^ self classForTestResult signalFailureWith: exampleResult gtDisplayString  ] ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:58'!
runExampleContainers: aCollection
	| allExamples |
	(allExamples := aCollection gather: [ :container | container gtExamplesContained ])
		do: [ :example | self runExample: example ]
		displayingProgress: 'Running ' , allExamples size asString , ' Examples'! !

!GTExampleTestCase methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 11:17'!
testExamples
	" default testcase "
	self runExampleContainers: (self packages: self packageNames)! !

RBBlockLintRule subclass: #GTExampleNotDefinedRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleNotDefinedRule commentStamp: 'AndreiChis 1/22/2016 16:20' prior: 0!
Every class that defines a custom extension for GTSpotter or GTInspector should define also, at least, a GTExample.!

!GTExampleNotDefinedRule class methodsFor: 'testing' stamp: 'AndreiChis 1/22/2016 15:46'!
checksClass
	^ true! !

!GTExampleNotDefinedRule class methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:02'!
uniqueIdentifierName
	"This number should be unique and should change only when the rule completely change semantics"
	
	^'GTExampleNotDefinedRule'! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:56'!
basicCheck: aClass 
	^ (self containsCustomExtension: aClass) 
		and: [ (self containsGTExample: aClass) not ]! !

!GTExampleNotDefinedRule methodsFor: 'private' stamp: 'AndreiChis 1/22/2016 16:15'!
check: aClass forCriticsDo: aCriticBlock ifNone: alternativeBlock
	(self basicCheck: aClass)
		ifTrue:  [ aCriticBlock cull: (
			(ReMissingMethodCritic
				for: aClass
				by: self
				class: aClass class
				selector: #gtExample) beShouldBeImplemented )  ]
		ifFalse: alternativeBlock ! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:10'!
checkClass: aClass 
	(self basicCheck: aClass) ifTrue: [
		result
			addClass: aClass ]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:09'!
containsCustomExtension: aClass

	^ aClass methods anySatisfy: [:aMethod | 
		aMethod pragmas anySatisfy: [ :aPragma |
			aPragma keyword = GTInspector extensionsPragma or: [
				aPragma keyword = GTSpotter extensionsPragma ]]]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:59'!
containsGTExample: aClass

	^ aClass class methods anySatisfy: [:aMethod | 
			aMethod pragmas 
				anySatisfy: [ :pragma | pragma isGTExamplePragma ]]! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
group
	^ 'Design Flaws'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:00'!
name
	^ 'Class defines a custom GT extension but no example'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
resultClass
	^ RBClassEnvironment! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
severity
	^ #information! !

!RBBrowserEnvironment methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/6/2016 21:17'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methodsDo: [ :each | 
		each isGTExampleMethod ifTrue: [ 
			each gtExample ifNotNil: [ :example | 
				examples add: example ] ] ].
	^ examples! !

!SubscriptionRegistry methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 11/29/2015 15:27'!
subscriptions
	^ subscriptions! !

Announcement subclass: #GTExamplesModifiedAnnouncement
	instanceVariableNames: 'announcement added removed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Announcements'!

!GTExamplesModifiedAnnouncement commentStamp: 'TudorGirba 8/21/2016 07:38' prior: 0!
I am an announcement announcing that an example method was modified.

I am particularly useful for allowing tools to refresh their state if something happens with an example.!

!GTExamplesModifiedAnnouncement methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:25'!
addExample: anExample
	anExample ifNil: [ ^ self ].
	added add: anExample! !

!GTExamplesModifiedAnnouncement methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:37'!
addExamples: aCollection
	aCollection ifNil: [ ^ self ].
	aCollection do: [ :value | self addExample: value ]! !

!GTExamplesModifiedAnnouncement methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 19:25'!
announcement
	^ announcement! !

!GTExamplesModifiedAnnouncement methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 19:25'!
announcement: anObject
	announcement := anObject! !

!GTExamplesModifiedAnnouncement methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 19:31'!
initialize
	super initialize.
	
	added := OrderedCollection new: 100.
	removed := OrderedCollection new: 100.! !

!GTExamplesModifiedAnnouncement methodsFor: 'testing' stamp: 'StefanReichhart 7/5/2016 11:15'!
isEmpty
	^ added isEmptyOrNil 
		or: [ removed isEmptyOrNil ]! !

!GTExamplesModifiedAnnouncement methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:25'!
removeExample: anExample
	anExample ifNil: [ ^ self ].
	removed add: anExample! !

!GTExamplesModifiedAnnouncement methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:37'!
removeExamples: aCollection
	aCollection ifNil: [ ^ self ].
	aCollection do: [ :value | self removeExample: value ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 16:10'!
asGTExampleSubject
	^ GTExampleMethodSubject fromClass: self methodClass selector: self selector! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:37'!
gtExample
	self gtExamplesFactory exampleMethod: self do: [ :example | ^ example ].
	^ nil! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 5/29/2016 12:22'!
gtExampleValue
	^ self gtExample returnValue! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/4/2016 15:18'!
gtExamples
	^ self gtExample 
		ifNotNil: [ :example | { example } ]
		ifNil: [ {  } ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/6/2016 15:00'!
gtExamplesFactory
	^ self methodClass gtExamplesFactory! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/14/2016 16:16'!
gtValueWithReceiver: aReceiver possibleArguments: anArray 
	^ anArray size = self numArgs 
		ifTrue: [ self valueWithReceiver: aReceiver arguments: anArray  ]
		ifFalse: [ self valueWithReceiver: aReceiver arguments: (anArray copyFrom: 1 to: self numArgs) ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:32'!
isGTExampleMethod
	^ self pragmas anySatisfy: [ :each | each isGTExamplePragma ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/4/2016 18:58'!
isGTReservedMethod
	^ #( gtExamplesSubjects gtExamplesFactory gtExample gtExamples ) 
			anySatisfy: [ :selector | selector = self selector ]! !

!Symbol methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 15:47'!
asGTExampleSubject
	^ GTExampleKeywordSubject fromSymbol: self! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/11/2016 16:29'!
asGTExampleSubject
	^ GTExampleClassSubject new 
		theClassName: self name;
		theClass: self;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:13'!
gtExamples
	^ self gtExamplesFactory gtExamplesContained! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:58'!
gtExamplesFactory
	^ GTExampleFactory new
		source: self;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/15/2016 13:11'!
gtExamplesSubjects
	^ { self }! !

!Text methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 15:46'!
asGTExampleSubject
	^ GTExampleTextSubject fromText: self! !

Error subclass: #GTExampleError
	instanceVariableNames: 'example signaledException'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleError commentStamp: 'TudorGirba 8/21/2016 07:45' prior: 0!
I am an abstract error induced by a signaledException related to a specific example.

My subclasses define the specific error.!

GTExampleError subclass: #GTExampleDependencyError
	instanceVariableNames: 'expected defined invalid'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDependencyError commentStamp: 'TudorGirba 8/21/2016 14:51' prior: 0!
I represent a static error induced by a wrong declaration of an example. There are two cases when I am triggered:
	- when the amount of method arguments are not matching the amount of dependency definitions 
	- when the example dependencies do not exist !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:24'!
defined
	^ defined! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:24'!
defined: anObject
	defined := anObject! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:18'!
expected
	^ expected! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:18'!
expected: anObject
	expected := anObject! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:22'!
invalid
	^ invalid! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 7/14/2016 18:22'!
invalid: anObject
	invalid := anObject! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example
	^ example! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example: anObject
	example := anObject! !

!GTExampleError methodsFor: 'testing' stamp: 'StefanReichhart 7/11/2016 13:08'!
isGTExampleError
	^ true! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException
	^ signaledException! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException: anObject
	signaledException := anObject! !

GTExampleError subclass: #GTExamplePragmaError
	instanceVariableNames: 'pragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExamplePragmaError commentStamp: 'TudorGirba 8/21/2016 14:51' prior: 0!
I am an abstract error related to the way the pragmas are used within an example method.!

GTExamplePragmaError subclass: #GTExampleDeclaredClassMissing
	instanceVariableNames: 'classNamed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed
	^ classNamed! !

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed: anObject
	classNamed := anObject! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma
	^ pragma! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma: anObject
	pragma := anObject! !

GTExamplePragmaError subclass: #GTExampleSubjectError
	instanceVariableNames: 'subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject
	^ subject! !

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject: anObject
	subject := anObject! !

GTExampleError subclass: #GTExampleRecursionDetected
	instanceVariableNames: 'resolved next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleRecursionDetected commentStamp: 'TudorGirba 8/21/2016 14:55' prior: 0!
I represent an error that occurs when we have cycles in static example dependencies. The cycles can be either direct or indirect. For example,  these configurations induces an error:
- a depends-on b, b depends-on a
- a depends-on b, b depends-on c, c depends-on a
	!

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next
	^ next! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next: anObject
	next := anObject! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved
	^ resolved! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved: anObject
	resolved := anObject! !

Error subclass: #GTExampleExpectedExceptionNotRaised
	instanceVariableNames: 'expectedError exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception
	^ exception! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception: anObject
	exception := anObject! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError
	^ expectedError! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError: anObject
	expectedError := anObject! !

Error subclass: #GTExampleUnexpectedExceptionRaised
	instanceVariableNames: 'unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleUnexpectedExceptionRaised methodsFor: 'UI' stamp: 'StefanReichhart 4/10/2016 11:14'!
gtDisplayOn: aStream
	super gtDisplayOn: aStream.
	aStream nextPutAll: ' ('.
	self unexpectedError gtDisplayOn: aStream.
	aStream nextPutAll: ')'.! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError
	^ unexpectedError! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError: anObject
	unexpectedError := anObject! !

CustomHelp subclass: #GTExampleHelp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Help'!

!GTExampleHelp commentStamp: 'TudorGirba 8/19/2016 23:55' prior: 0!
I hold the help for the examples engine.!

!GTExampleHelp class methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 18:09'!
bookName 
	^'GT Examples'! !

!GTExampleHelp class methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 18:09'!
key
	^ self name! !

!GTExampleHelp class methodsFor: 'accessing' stamp: 'StefanReichhart 7/21/2016 18:10'!
pages 
	^ #( pragmas )! !

!GTExampleHelp class methodsFor: 'documentation' stamp: 'StefanReichhart 7/21/2016 18:20'!
pragmas
	^ HelpTopic 
		title: 'Pragmas'
		contents: self pragmasContents
		yourself! !

!GTExampleHelp class methodsFor: 'documentation' stamp: 'StefanReichhart 7/21/2016 18:25'!
pragmasContents
	^ String streamContents: [ :stream |
		((Pragma allNamed: #gtExamplePragma in: GTExample) 
			sort: [ :a :b | a method selector <= b method selector ]) 
			do: [ :each |
				stream nextPutAll: '<'.
				stream nextPutAll: (each method sourceCode copyUpTo: Character cr).
				stream nextPutAll: '>'.
				stream cr.
				each method pragmas do: [ :pragma | 
					pragma == each ifFalse: [ 
						stream nextPutAll: (String streamContents: [ :argStream |
							pragma arguments 
								do: [ :arg | argStream nextPutAll: arg asString ]
								separatedBy: [ argStream cr ] ]) ] ].
				stream cr; cr ] ]! !

!SequenceableCollection methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 12:23'!
gtExampleSubstrings
	^ (self gather: [ :each | each gtExampleSubstrings ])
		reject: [ :each | each isEmptyOrNil ]! !

!RGMethodDefinition methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 8/13/2016 16:09'!
asGTExampleSubject
	^ GTExampleMethodSubject fromClass: self realClass selector: self selector! !

!Pragma methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/9/2015 19:54'!
isGTExamplePragma
	<toBeDeprecated: #gtExample:>
	^ self keyword = #gtExample
		or: [ self keyword = #gtExample: ]! !
